/*
 * PrettyFaces is an OpenSource JSF library to create bookmarkable URLs.
 * Copyright (C) 2010 - Lincoln Baxter, III <lincoln@ocpsoft.com> This program
 * is free software: you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version. This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see the file COPYING.LESSER
 * or visit the GNU website at <http://www.gnu.org/licenses/>.
 */

package com.ocpsoft.pretty.util;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;

import com.ocpsoft.pretty.PrettyException;
import com.ocpsoft.pretty.config.mapping.RequestParameter;

/**
 * @author Lincoln Baxter, III <lincoln@ocpsoft.com>
 */
public class QueryString
{
    private final Map<String, List<String>> parameters = new HashMap<String, List<String>>();

    /**
     * Factory methods
     */

    public static QueryString build(final Map<String, String[]> params)
    {
        QueryString queryString = new QueryString();
        queryString.addParameters(params);
        return queryString;
    }

    public static <T extends RequestParameter> QueryString build(final List<T> params)
    {
        QueryString queryString = new QueryString();
        queryString.addParameters(params);
        return queryString;
    }

    public static QueryString build(final String url)
    {
        QueryString queryString = new QueryString();
        queryString.addParameters(url);
        return queryString;
    }

    /**
     * Work methods
     */
    public String getParameter(final String name)
    {
        List<String> values = parameters.get(name);
        if (values == null)
        {
            return null;
        }

        if (values.size() == 0)
        {
            return "";
        }

        return values.get(0);
    }

    public String[] getParameterValues(final String name)
    {
        List<String> values = parameters.get(name);
        if (values == null)
        {
            return null;
        }

        return values.toArray(new String[values.size()]);
    }

    public Enumeration<String> getParameterNames()
    {
        return Collections.enumeration(parameters.keySet());
    }

    public Map<String, String[]> getParameterMap()
    {
        Map<String, String[]> map = new TreeMap<String, String[]>();
        for (Map.Entry<String, List<String>> entry : parameters.entrySet())
        {
            List<String> list = entry.getValue();
            String[] values;
            if (list == null)
            {
                values = null;
            }
            else
            {
                values = list.toArray(new String[list.size()]);
            }
            map.put(entry.getKey(), values);
        }
        return map;
    }

    /**
     * Add Parameters
     */

    public <T extends RequestParameter> void addParameters(final List<T> params)
    {
        for (RequestParameter rp : params)
        {
            String value = rp.getValue();
            String name = rp.getName();

            if (!parameters.containsKey(name))
            {
                ArrayList<String> values = new ArrayList<String>();
                if (value != null)
                {
                    values.add(value);
                }
                parameters.put(name, values);
            }
            else
            {
                parameters.get(name).add(value);
            }
        }
    }

    public void addParameters(final Map<String, String[]> params)
    {
        if (params != null)
        {
            for (Entry<String, String[]> entry : params.entrySet())
            {
                List<String> values = null;
                if (entry.getValue() != null)
                {
                    values = Arrays.asList(entry.getValue());
                }
                parameters.put(entry.getKey(), values);
            }
        }
    }

    /**
     * Add all query parameters from the given URL. URL must contain the ?
     * delimiter.
     * 
     * @param url
     */
    public void addParameters(String url)
    {
        if (url != null && !"".equals(url))
        {
            int pos = url.indexOf('?');
            if (pos >= 0 && url.length() > 1)
            {
                url = url.substring(pos + 1);

                String pairs[] = url.split("&");
                for (String pair : pairs)
                {
                    String name;
                    String value;
                    pos = pair.indexOf('=');
                    // for "n=", the value is "", for "n", the value is null
                    if (pos == -1)
                    {
                        name = pair;
                        value = null;
                    }
                    else
                    {
                        try
                        {
                            name = URLDecoder.decode(pair.substring(0, pos), "UTF-8");
                            value = URLDecoder.decode(pair.substring(pos + 1, pair.length()), "UTF-8");
                        }
                        catch (UnsupportedEncodingException e)
                        {
                            throw new PrettyException(
                                    "UTF-8 encoding not supported. Something is seriously wrong with your environment.");
                        }
                    }
                    List<String> list = parameters.get(name);
                    if (list == null)
                    {
                        list = new ArrayList<String>();
                        parameters.put(name, list);
                    }
                    list.add(value);
                }
            }
        }
        return;
    }

    /**
     * Convert the current parameters to a valid query string.
     * 
     * @return If parameters exist, return a valid query string with leading '?'
     *         character. If no parameters exist, return an empty string.
     */
    public String toQueryString()
    {
        try
        {
            StringBuffer result = new StringBuffer();

            if (null != parameters && !parameters.isEmpty())
            {
                result.append("?");
                Iterator<Entry<String, List<String>>> iterator = parameters.entrySet().iterator();
                while (iterator.hasNext())
                {
                    Entry<String, List<String>> entry = iterator.next();
                    String key = entry.getKey();
                    List<String> values = entry.getValue();

                    if (key != null && !"".equals(key))
                    {
                        key = URLEncoder.encode(key, "UTF-8");
                        result.append(key);

                        if (values != null && !values.isEmpty())
                        {
                            for (int i = 0; i < values.size(); i++)
                            {
                                String value = values.get(i);
                                if (value != null && !"".equals(value))
                                {
                                    value = URLEncoder.encode(value, "UTF-8");
                                    result.append("=" + value);
                                }
                                else if (value != null && "".equals(value))
                                {
                                    result.append("=");
                                }

                                if (i < values.size() - 1)
                                {
                                    result.append("&" + key);
                                }
                            }
                        }
                    }

                    if (iterator.hasNext())
                    {
                        result.append("&");
                    }
                }
            }

            return result.toString();
        }
        catch (UnsupportedEncodingException e)
        {
            throw new PrettyException("Error building query string.", e);
        }
    }

    @Override
    public String toString()
    {
        return "QueryString [ query=" + toQueryString() + "]";
    }
}